unit SecretDlgs;

{-------------------------------------------------------------------------------
  SecretDlgs.pas
  nove dialogove boxy
  2003 (c) manison
-------------------------------------------------------------------------------}

interface

uses Windows, Forms, SysUtils;

type
  TPickIconDlgW = function (hwnd: THandle; pszIconPath: PWideChar;
    cchIconPath: Cardinal; var piIconIndex: Integer): LongBool; stdcall;
  TPickIconDlgA = function (hwnd: THandle; pszIconPath: PChar;
    cchIconPath: Cardinal; var piIconIndex: Integer): LongBool; stdcall;

function PickIconDlg(hwnd: THandle; var IconPath: string;
  var piIconIndex: Integer): Boolean;

const
  SID_IProgressDialog = '{EBBC7C04-315E-11D2-B62F-006097DF5BD4}';
  CLSID_ProgressDialog: TGUID = '{F8383852-FCD3-11D1-A6B9-006097DF5BD4}';
  IID_IProgressDialog: TGUID = SID_IProgressDialog;

  PROGDLG_NORMAL        =  $00000000;
  PROGDLG_MODAL         =  $00000001;
  PROGDLG_AUTOTIME      =  $00000002;
  PROGDLG_NOTIME        =  $00000004;
  PROGDLG_NOMINIMIZE    =  $00000008;
  PROGDLG_NOPROGRESSBAR =  $00000010;

  PDTIMER_RESET       = $00000001;

type
  LPCVOID = Pointer;

  IProgressDialog = interface(IUnknown)
  [SID_IProgressDialog]
    function StartProgressDialog(hwndParent: HWND; punkEnableModless: IUnknown;
      dwFlags: DWORD; pvReserved: LPCVOID): HResult; stdcall;
    function StopProgressDialog: HResult; stdcall;
    function SetTitle(pwzTitle: LPCWSTR): HResult; stdcall;
    function SetAnimation(hInstAnimation: THandle; idAnimation: UINT): HResult; stdcall;
    function HasUserCancelled: BOOL; stdcall;
    function SetProgress(dwCompleted, dwTotal: DWORD): HResult; stdcall;
    function SetProgress64(ullCompleted, ullTotal: Int64): HResult; stdcall;
    function SetLine(dwLine: DWORD; pwzString: LPCWSTR; fCompactPath: BOOL;
      pvReserved: LPCVOID): HResult; stdcall;
    function SetCancelMsg(pwzCancelMsg: LPCWSTR; pvReserved: LPCVOID): HResult; stdcall;
    function Timer(dwTimerAction: DWORD; pvReserved: LPCVOID): HResult; stdcall;
  end;

  TRunFileDlgA = procedure (hwnd: HWND; hIcon: HICON;
    lpszDirectory, lpszTitle, lpszDescription: PAnsiChar; dwFlags: Cardinal); stdcall;
  TRunFileDlgW = procedure (hwnd: HWND; hIcon: HICON;
    lpszDirectory, lpszTitle, lpszDescription: PWideChar; dwFlags: Cardinal); stdcall;

function RunFileDlg(hwnd: HWND; hIcon: HICON; Directory, Title, Description: string; dwFlags: Cardinal): Boolean;

const
  RFF_NOBROWSE      = $0001;  { bez tlacitka Prochazet }
  RFF_NODEFAULT     = $0002;  { na zacatku neni zvolena zadna polozka }
  RFF_CALCDIRECTORY = $0004;  { vytvori pracovni adresar na zaklade jmena souboru }
  RFF_NOLABEL       = $0008;  { bez napisu Otevrit }

  RFN_VALIDATE = -510;  { kod WM_NOTIFY }

  { navratove hodnoty WM_NOTIFY }
  RF_OK     = $0000;  { aplikace bude spustena }
  RF_CANCEL = $0001;  { zavre dialog bez spusteni programu }
  RF_RETRY  = $0002;  { aplikace nebude spustena ale necha otevrene dialogove okno }

  RF_MAX_DIRECTORY = MAX_PATH;
  RF_MAX_TITLE = 64;
  RF_MAX_DESCRIPTION = 128;

type
  TNMRunFileDlgW = packed record
    hdr: TNMHdr;
    lpszFile: PWideChar;
    lpszDirectory: PWideChar;
    nShow: Integer;
  end;
  PNMRunFileDlgW = ^TNMRunFileDlgW;

  TNMRunFileDlgA = packed record
    hdr: TNMHdr;
    lpszFile: PAnsiChar;
    lpszDirectory: PAnsiChar;
    nShow: Integer;
  end;
  PNMRunFileDlgA = ^TNMRunFileDlgA;

implementation

function PickIconDlg(hwnd: THandle; var IconPath: string;
  var piIconIndex: Integer): Boolean;
var
  PickIconDlgFunc: Pointer;
  hShell: THandle;
  pszIconPath: Pointer;
begin
  hShell := LoadLibrary('shell32.dll');
  if hShell = 0 then begin
    Result := False;
    RaiseLastWin32Error;
    Exit;
  end;

  PickIconDlgFunc := GetProcAddress(hShell, 'PickIconDlg');

  if (PickIconDlgFunc = nil)
    then PickIconDlgFunc := GetProcAddress(hShell, PChar(62));

  if PickIconDlgFunc <> nil then begin
    if GetVersion < $80000000 then begin // WinNT/2k/XP
      GetMem(pszIconPath, SizeOf(WideChar)*MAX_PATH);
      StringToWideChar(IconPath, pszIconPath, MAX_PATH);
      Result := TPickIconDlgW(PickIconDlgFunc)(hwnd, pszIconPath, MAX_PATH, piIconIndex);
      IconPath := PWideChar(pszIconPath);
      FreeMem(pszIconPath);
    end
    else begin  // Win9x/Me
      GetMem(pszIconPath, SizeOf(Char)*MAX_PATH);
      StrPLCopy(pszIconPath, IconPath, MAX_PATH);
      Result := TPickIconDlgA(PickIconDlgFunc)(hwnd, pszIconPath, MAX_PATH, piIconIndex);
      IconPath := PChar(pszIconPath);
      FreeMem(pszIconPath);
    end
  end
  else begin
    Result := False;
    RaiseLastWin32Error;
  end;
  FreeLibrary(hShell);
end;

function RunFileDlg(hwnd: HWND; hIcon: HICON; Directory, Title, Description: string; dwFlags: Cardinal): Boolean;
var
  RunFileDlgFunc: Pointer;
  hShell: THandle;
begin
  hShell := LoadLibrary('shell32.dll');
  if hShell = 0 then begin
    Result := False;
    RaiseLastWin32Error;
    Exit;
  end;
  RunFileDlgFunc := GetProcAddress(hShell, PChar(61));
  if RunFileDlgFunc = nil then begin
    Result := False;
    RaiseLastWin32Error;
  end
  else begin
    Result := True;
    if GetVersion < $80000000 then  // WinNT/2k/XP
      TRunFileDlgW(RunFileDlgFunc)(hwnd, hIcon, PWideChar(WideString(Directory)),
        PWideChar(WideString(Title)), PWideChar(WideString(Description)), dwFlags)
    else
      TRunFileDlgA(RunFileDlgFunc)(hwnd, hIcon, PChar(Directory),
        PChar(Title), PChar(Description), dwFlags);
  end;
  FreeLibrary(hShell);
end;

end.
