unit main;

{-------------------------------------------------------------------------------
  main.pas
  hlavni unit demo projektu 'Skryte funkce Windows - Dialogova okna'
  2003 (c) manison
-------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Classes, Forms, Controls, StdCtrls, Dialogs,
  ExtCtrls, ComObj, SecretDlgs;

type
  TfmWinSecrets = class(TForm)
    btIconDlg: TButton;
    btProgressDlgShow: TButton;
    btProgressDlgClose: TButton;
    edLine1: TEdit;
    edLine3: TEdit;
    edLine2: TEdit;
    chCompact3: TCheckBox;
    chCompact2: TCheckBox;
    chCompact1: TCheckBox;
    chModal: TCheckBox;
    chAutoTime: TCheckBox;
    chNoTime: TCheckBox;
    chNoProgress: TCheckBox;
    chNoMinim: TCheckBox;
    Timer: TTimer;
    btRunDlg: TButton;
    edRDTitle: TEdit;
    edRDDescr: TEdit;
    chNoLabel: TCheckBox;
    chNoDefault: TCheckBox;
    chNoBrowse: TCheckBox;
    rbRetry: TRadioButton;
    rbCancel: TRadioButton;
    rbRun: TRadioButton;
    lbFile: TLabel;
    edTitle: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Bevel1: TBevel;
    Label9: TLabel;
    Bevel2: TBevel;
    Label10: TLabel;
    Bevel3: TBevel;
    procedure btIconDlgClick(Sender: TObject);
    procedure btProgressDlgShowClick(Sender: TObject);
    procedure btProgressDlgCloseClick(Sender: TObject);
    procedure chAutoTimeClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure btRunDlgClick(Sender: TObject);
  private
    procedure EnableGroup(Enable: Boolean);
    procedure WMNotify(var Msg: TWMNotify); message WM_NOTIFY;
  end;

var
  fmWinSecrets: TfmWinSecrets;
  ProgressDlgIntf: IProgressDialog;
  Completed: Integer;

implementation

{$R *.dfm}
{$R animation.res}

procedure TfmWinSecrets.btIconDlgClick(Sender: TObject);
var
  iIcon: Integer;
  Path: string;
begin
  iIcon := 0;
  Path := Application.ExeName;

  if PickIconDlg(Handle, Path, iIcon) then begin
    ShowMessage(Format('Byla vybrna ikona . %d ze souboru %s.', [iIcon, Path]));
  end
  else
    ShowMessage('Vbr ikony byl zruen.');
    
end;

procedure TfmWinSecrets.btProgressDlgShowClick(Sender: TObject);
const
  aAutoTime: array [Boolean] of Cardinal = (0, PROGDLG_AUTOTIME);
  aModal: array [Boolean] of Cardinal = (PROGDLG_NORMAL, PROGDLG_MODAL);
  aNoTime: array [Boolean] of Cardinal = (0, PROGDLG_NOTIME);
  aNoMinimize: array [Boolean] of Cardinal = (0, PROGDLG_NOMINIMIZE);
  aNoProgress: array [Boolean] of Cardinal = (0, PROGDLG_NOPROGRESSBAR);
var
  unk: IUnknown;
begin
  try
    unk := CreateComObject(CLSID_ProgressDialog);
  except
    ShowMessage('Nelze vytvoit objekt rozhran IProgressDialog. Pravdpodobn '+
      'nemte potebnou verzi Windows.');
    Exit;
  end;
  if Supports(unk, IProgressDialog, ProgressDlgIntf) then begin
    ProgressDlgIntf._AddRef;
    EnableGroup(True);
    with ProgressDlgIntf do begin
      SetTitle(PWideChar(WideString(edTitle.Text)));
      SetCancelMsg('Pokejte, prosm, na dokonen operace.', nil);
      SetAnimation(HInstance, 1);
      StartProgressDialog(Handle, nil,
        aAutoTime[chAutoTime.Checked] or aModal[chModal.Checked] or aNoTime[chNoTime.Checked] or
        aNoMinimize[chNoMinim.Checked] or aNoProgress[chNoProgress.Checked], nil);
      SetLine(1, PWideChar(WideString(edLine1.Text)), chCompact1.Checked, nil);
      SetLine(2, PWideChar(WideString(edLine2.Text)), chCompact2.Checked, nil);
      if not chAutoTime.Checked then SetLine(3, PWideChar(WideString(edLine3.Text)), chCompact3.Checked, nil);
      Completed := 0;
    end;
    Timer.Enabled := True;
  end
  else
    ShowMessage('Objekt nepodporuje rozhran IProgressDialog.');
end;

procedure TfmWinSecrets.EnableGroup(Enable: Boolean);
var
  i: Integer;
begin
  for i := 0 to ComponentCount-1 do
    if Components[i].Tag = 1 then TControl(Components[i]).Enabled := Enable
    else if Components[i].Tag = 2 then TControl(Components[i]).Enabled := not Enable;
  if not Enable then chAutoTimeClick(nil);
end;

procedure TfmWinSecrets.btProgressDlgCloseClick(Sender: TObject);
begin
  Timer.Enabled := False;
  with ProgressDlgIntf do begin
    StopProgressDialog;
    _Release;
  end;
  EnableGroup(False);
end;

procedure TfmWinSecrets.chAutoTimeClick(Sender: TObject);
begin
  edLine3.Enabled := not chAutoTime.Checked;
  chCompact3.Enabled := edLine3.Enabled;
end;

procedure TfmWinSecrets.FormCreate(Sender: TObject);
begin
  EnableGroup(False);
end;

procedure TfmWinSecrets.TimerTimer(Sender: TObject);
begin
  if ProgressDlgIntf.HasUserCancelled then begin
    Sleep(2000);
    btProgressDlgCloseClick(nil);
  end;
  Inc(Completed);
  ProgressDlgIntf.SetProgress(Completed, 100);
  if Completed = 100 then btProgressDlgCloseClick(nil);
end;

procedure TfmWinSecrets.btRunDlgClick(Sender: TObject);
const
  aNoBrowse: array [Boolean] of Cardinal = (0, RFF_NOBROWSE);
  aNoDefault: array [Boolean] of Cardinal = (0, RFF_NODEFAULT);
  aNoLabel: array [Boolean] of Cardinal = (0, RFF_NOLABEL);
begin
  RunFileDlg(Handle, Application.Icon.Handle, '', edRDTitle.Text, edRDDescr.Text,
    RFF_CALCDIRECTORY or aNoBrowse[chNoBrowse.Checked] or
    aNoDefault[chNoDefault.Checked] or aNoLabel[chNoLabel.Checked]);
end;

procedure TfmWinSecrets.WMNotify(var Msg: TWMNotify);
begin
  if Msg.NMHdr^.code = RFN_VALIDATE then begin
    if rbCancel.Checked then Msg.Result := RF_CANCEL
    else if rbRetry.Checked then Msg.Result := RF_RETRY
    else Msg.Result := RF_OK;
    if GetVersion < $80000000 then begin  // WinNT/2k/XP
      with PNMRunFileDlgW(Msg.NMHdr)^ do begin
        lbFile.Caption := lpszFile;
      end;
    end
    else begin
      with PNMRunFileDlgA(Msg.NMHdr)^ do begin
        lbFile.Caption := lpszFile;
      end;
    end;
  end
  else
    inherited;
end;

end.
